@@ -438,7 +438,8 @@ actions_plugin_size_changed (XfcePanelPlugin *panel_plugin,
       box = gtk_bin_get_child (GTK_BIN (plugin));
       if (box != NULL)
         {
-          if (plugin->invert_orientation)
+          if (plugin->invert_orientation !=
+              (xfce_panel_plugin_get_mode (panel_plugin) == XFCE_PANEL_PLUGIN_MODE_DESKBAR))
             {
               children = gtk_container_get_children (GTK_CONTAINER (box));
               if (G_UNLIKELY (children == NULL))
@@ -1092,12 +1093,15 @@ actions_plugin_pack_idle (gpointer data)
   if (plugin->items == NULL)
     plugin->items = actions_plugin_default_array ();
 
-  orientation = xfce_panel_plugin_get_orientation (XFCE_PANEL_PLUGIN (plugin));
-
   allowed_types = actions_plugin_actions_allowed ();
 
   if (plugin->type == APPEARANCE_TYPE_BUTTONS)
     {
+      if (xfce_panel_plugin_get_mode (XFCE_PANEL_PLUGIN (plugin)) == XFCE_PANEL_PLUGIN_MODE_VERTICAL)
+        orientation = GTK_ORIENTATION_VERTICAL;
+      else
+        orientation = GTK_ORIENTATION_HORIZONTAL;
+
       if (plugin->invert_orientation)
         orientation = !orientation;
       box = xfce_hvbox_new (orientation, FALSE, 0);
@@ -1113,7 +1117,8 @@ actions_plugin_pack_idle (gpointer data)
 
           /* skip separators when packing buttons in the opposite
            * orientation */
-          if (plugin->invert_orientation
+          if (plugin->invert_orientation !=
+              (xfce_panel_plugin_get_mode (XFCE_PANEL_PLUGIN (plugin)) == XFCE_PANEL_PLUGIN_MODE_DESKBAR)
               && g_strcmp0 (name + 1, "separator") == 0)
             continue;
 
@@ -1247,6 +1252,11 @@ actions_plugin_menu (GtkWidget     *button,
   ActionType    allowed_types;
 
   panel_return_if_fail (XFCE_IS_ACTIONS_PLUGIN (plugin));
+  panel_return_if_fail (button != NULL);
+
+  /* do not popup the menu if the button is being toggled off */
+  if (!gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button)))
+    return;
 
   if (plugin->menu == NULL)
     {
@@ -1255,6 +1265,7 @@ actions_plugin_menu (GtkWidget     *button,
           G_CALLBACK (actions_plugin_menu_deactivate), button);
       g_object_add_weak_pointer (G_OBJECT (plugin->menu), (gpointer) &plugin->menu);
 
+      size = DEFAULT_ICON_SIZE;
       if (gtk_icon_size_lookup (menu_icon_size, &w, &h))
         size = MIN (w, h);
 
